@@ -36,8 +36,13 @@ sa = nk.sampler.MetropolisLocal(machine=ma, n_chains=2)
 op = nk.optimizer.Sgd(ma, learning_rate=0.1)
 
 # Create the optimization driver
-gs = nk.Vmc(
-    hamiltonian=ha, sampler=sa, optimizer=op, n_samples=1000, sr=None, n_discard=None
+gs = nk.VMC(
+    hamiltonian=ha,
+    sampler=sa,
+    optimizer=op,
+    n_samples=1000,
+    sr=None,
+    n_discard_per_chain=None,
 )
 
 # The first iteration is slower because of start-up jit times
