@@ -65,6 +65,7 @@ def Grid(
     *,
     length: Sequence[int] = None,
     pbc: Union[bool, Sequence[bool]] = True,
+    **kwargs,
 ) -> Lattice:
     """
     Constructs a hypercubic lattice given its extent in all dimensions.
@@ -77,6 +78,8 @@ def Grid(
              This parameter can also be a list of booleans with same length as
              the parameter `length`, in which case each dimension will have
              PBC/OBC depending on the corresponding entry of `pbc`.
+        kwargs: Additional keyword arguments are passed on to the constructor of
+            :ref:`netket.graph.Lattice`.
 
     Examples:
         Construct a 5x10 square lattice with periodic boundary conditions:
@@ -118,10 +121,11 @@ def Grid(
         extent=extent,
         pbc=pbc,
         point_group=lambda: _grid_point_group(extent, pbc),
+        **kwargs,
     )
 
 
-def Hypercube(length: int, n_dim: int = 1, *, pbc: bool = True) -> Lattice:
+def Hypercube(length: int, n_dim: int = 1, *, pbc: bool = True, **kwargs) -> Lattice:
     r"""Constructs a hypercubic lattice with equal side length in all dimensions.
     Periodic boundary conditions can also be imposed.
 
@@ -130,6 +134,8 @@ def Hypercube(length: int, n_dim: int = 1, *, pbc: bool = True) -> Lattice:
         n_dim: Dimension of the hypercube; must be at least 1.
         pbc: Whether the hypercube should have periodic boundary conditions
             (in all directions)
+        kwargs: Additional keyword arguments are passed on to the constructor of
+            :ref:`netket.graph.Lattice`.
 
     Examples:
          A 10x10x10 cubic lattice with periodic boundary conditions can be
@@ -143,10 +149,10 @@ def Hypercube(length: int, n_dim: int = 1, *, pbc: bool = True) -> Lattice:
     if not isinstance(length, int) or length <= 0:
         raise TypeError("Argument `length` must be a positive integer")
     length_vector = [length] * n_dim
-    return Grid(length_vector, pbc=pbc)
+    return Grid(length_vector, pbc=pbc, **kwargs)
 
 
-def Cube(length: int, *, pbc: bool = True) -> Lattice:
+def Cube(length: int, *, pbc: bool = True, **kwargs) -> Lattice:
     """Constructs a cubic lattice of side `length`
     Periodic boundary conditions can also be imposed
 
@@ -154,6 +160,8 @@ def Cube(length: int, *, pbc: bool = True) -> Lattice:
         length: Side length of the cube; must always be >=1
         pbc: Whether the cube should have periodic boundary conditions
             (in all directions)
+        kwargs: Additional keyword arguments are passed on to the constructor of
+            :ref:`netket.graph.Lattice`.
 
     Examples:
         A 10×10×10 cubic lattice with periodic boundary conditions can be
@@ -164,10 +172,10 @@ def Cube(length: int, *, pbc: bool = True) -> Lattice:
         >>> print(g.n_nodes)
         1000
     """
-    return Hypercube(length, pbc=pbc, n_dim=3)
+    return Hypercube(length, pbc=pbc, n_dim=3, **kwargs)
 
 
-def Square(length: int, *, pbc: bool = True) -> Lattice:
+def Square(length: int, *, pbc: bool = True, **kwargs) -> Lattice:
     """Constructs a square lattice of side `length`
     Periodic boundary conditions can also be imposed
 
@@ -175,6 +183,8 @@ def Square(length: int, *, pbc: bool = True) -> Lattice:
         length: Side length of the square; must always be >=1
         pbc: Whether the square should have periodic boundary
             conditions (in both directions)
+        kwargs: Additional keyword arguments are passed on to the constructor of
+            :ref:`netket.graph.Lattice`.
 
     Examples:
         A 10x10 square lattice with periodic boundary conditions can be
@@ -185,16 +195,18 @@ def Square(length: int, *, pbc: bool = True) -> Lattice:
         >>> print(g.n_nodes)
         100
     """
-    return Hypercube(length, pbc=pbc, n_dim=2)
+    return Hypercube(length, pbc=pbc, n_dim=2, **kwargs)
 
 
-def Chain(length: int, *, pbc: bool = True) -> Lattice:
+def Chain(length: int, *, pbc: bool = True, **kwargs) -> Lattice:
     r"""Constructs a chain of `length` sites.
     Periodic boundary conditions can also be imposed
 
     Args:
         length: Length of the chain. It must always be >=1
         pbc: Whether the chain should have periodic boundary conditions
+        kwargs: Additional keyword arguments are passed on to the constructor of
+            :ref:`netket.graph.Lattice`.
 
     Examples:
         A 10 site chain with periodic boundary conditions can be
@@ -205,10 +217,12 @@ def Chain(length: int, *, pbc: bool = True) -> Lattice:
         >>> print(g.n_nodes)
         10
     """
-    return Hypercube(length, pbc=pbc, n_dim=1)
+    return Hypercube(length, pbc=pbc, n_dim=1, **kwargs)
 
 
-def BCC(extent: Sequence[int], *, pbc: Union[bool, Sequence[bool]] = True) -> Lattice:
+def BCC(
+    extent: Sequence[int], *, pbc: Union[bool, Sequence[bool]] = True, **kwargs
+) -> Lattice:
     """Constructs a BCC lattice of a given spatial extent.
     Periodic boundary conditions can also be imposed
     Sites are returned at the Bravais lattice points.
@@ -221,6 +235,9 @@ def BCC(extent: Sequence[int], *, pbc: Union[bool, Sequence[bool]] = True) -> La
              This parameter can also be a list of booleans with same length as
              the parameter `length`, in which case each dimension will have
              PBC/OBC depending on the corresponding entry of `pbc`.
+        kwargs: Additional keyword arguments are passed on to the constructor of
+            :ref:`netket.graph.Lattice`.
+
 
     Example:
         Construct a BCC lattice with 3×3×3 primitive unit cells:
@@ -233,10 +250,14 @@ def BCC(extent: Sequence[int], *, pbc: Union[bool, Sequence[bool]] = True) -> La
     basis = [[-0.5, 0.5, 0.5], [0.5, -0.5, 0.5], [0.5, 0.5, -0.5]]
     # determine if full point group is realised by the simulation box
     point_group = cubic.Oh() if np.all(pbc) and len(set(extent)) == 1 else None
-    return Lattice(basis_vectors=basis, extent=extent, pbc=pbc, point_group=point_group)
+    return Lattice(
+        basis_vectors=basis, extent=extent, pbc=pbc, point_group=point_group, **kwargs
+    )
 
 
-def FCC(extent: Sequence[int], *, pbc: Union[bool, Sequence[bool]] = True) -> Lattice:
+def FCC(
+    extent: Sequence[int], *, pbc: Union[bool, Sequence[bool]] = True, **kwargs
+) -> Lattice:
     """Constructs an FCC lattice of a given spatial extent.
     Periodic boundary conditions can also be imposed
     Sites are returned at the Bravais lattice points.
@@ -249,6 +270,9 @@ def FCC(extent: Sequence[int], *, pbc: Union[bool, Sequence[bool]] = True) -> La
              This parameter can also be a list of booleans with same length as
              the parameter `length`, in which case each dimension will have
              PBC/OBC depending on the corresponding entry of `pbc`.
+        kwargs: Additional keyword arguments are passed on to the constructor of
+            :ref:`netket.graph.Lattice`.
+
 
     Example:
         Construct an FCC lattice with 3×3×3 primitive unit cells:
@@ -261,11 +285,13 @@ def FCC(extent: Sequence[int], *, pbc: Union[bool, Sequence[bool]] = True) -> La
     basis = [[0, 0.5, 0.5], [0.5, 0, 0.5], [0.5, 0.5, 0]]
     # determine if full point group is realised by the simulation box
     point_group = cubic.Oh() if np.all(pbc) and len(set(extent)) == 1 else None
-    return Lattice(basis_vectors=basis, extent=extent, pbc=pbc, point_group=point_group)
+    return Lattice(
+        basis_vectors=basis, extent=extent, pbc=pbc, point_group=point_group, **kwargs
+    )
 
 
 def Diamond(
-    extent: Sequence[int], *, pbc: Union[bool, Sequence[bool]] = True
+    extent: Sequence[int], *, pbc: Union[bool, Sequence[bool]] = True, **kwargs
 ) -> Lattice:
     """Constructs a diamond lattice of a given spatial extent.
     Periodic boundary conditions can also be imposed.
@@ -281,6 +307,9 @@ def Diamond(
              This parameter can also be a list of booleans with same length as
              the parameter `length`, in which case each dimension will have
              PBC/OBC depending on the corresponding entry of `pbc`.
+        kwargs: Additional keyword arguments are passed on to the constructor of
+            :ref:`netket.graph.Lattice`.
+
 
     Example:
         Construct a diamond lattice with 3×3×3 primitive unit cells:
@@ -300,11 +329,12 @@ def Diamond(
         extent=extent,
         pbc=pbc,
         point_group=point_group,
+        **kwargs,
     )
 
 
 def Pyrochlore(
-    extent: Sequence[int], *, pbc: Union[bool, Sequence[bool]] = True
+    extent: Sequence[int], *, pbc: Union[bool, Sequence[bool]] = True, **kwargs
 ) -> Lattice:
     """Constructs a pyrochlore lattice of a given spatial extent.
     Periodic boundary conditions can also be imposed.
@@ -320,6 +350,9 @@ def Pyrochlore(
             This parameter can also be a list of booleans with same length as
             the parameter `length`, in which case each dimension will have
             PBC/OBC depending on the corresponding entry of `pbc`.
+        kwargs: Additional keyword arguments are passed on to the constructor of
+            :ref:`netket.graph.Lattice`.
+
 
     Example:
         Construct a pyrochlore lattice with 3×3×3 primitive unit cells:
@@ -339,11 +372,12 @@ def Pyrochlore(
         extent=extent,
         pbc=pbc,
         point_group=point_group,
+        **kwargs,
     )
 
 
 def _hexagonal_general(
-    extent, *, site_offsets=None, pbc: Union[bool, Sequence[bool]] = True
+    extent, *, site_offsets=None, pbc: Union[bool, Sequence[bool]] = True, **kwargs
 ) -> Lattice:
     basis = [[1, 0], [0.5, 0.75 ** 0.5]]
     # determine if full point group is realised by the simulation box
@@ -354,10 +388,11 @@ def _hexagonal_general(
         site_offsets=site_offsets,
         pbc=pbc,
         point_group=point_group,
+        **kwargs,
     )
 
 
-def Triangular(extent, *, pbc: Union[bool, Sequence[bool]] = True) -> Lattice:
+def Triangular(extent, *, pbc: Union[bool, Sequence[bool]] = True, **kwargs) -> Lattice:
     r"""Constructs a triangular lattice of a given spatial extent.
     Periodic boundary conditions can also be imposed
     Sites are returned at the Bravais lattice points.
@@ -370,6 +405,9 @@ def Triangular(extent, *, pbc: Union[bool, Sequence[bool]] = True) -> Lattice:
              This parameter can also be a list of booleans with same length as
              the parameter `length`, in which case each dimension will have
              PBC/OBC depending on the corresponding entry of `pbc`.
+        kwargs: Additional keyword arguments are passed on to the constructor of
+            :ref:`netket.graph.Lattice`.
+
 
     Example:
         Construct a triangular lattice with 3 × 3 unit cells:
@@ -379,10 +417,10 @@ def Triangular(extent, *, pbc: Union[bool, Sequence[bool]] = True) -> Lattice:
         >>> print(g.n_nodes)
         9
     """
-    return _hexagonal_general(extent, site_offsets=None, pbc=pbc)
+    return _hexagonal_general(extent, site_offsets=None, pbc=pbc, **kwargs)
 
 
-def Honeycomb(extent, *, pbc: Union[bool, Sequence[bool]] = True) -> Lattice:
+def Honeycomb(extent, *, pbc: Union[bool, Sequence[bool]] = True, **kwargs) -> Lattice:
     r"""Constructs a honeycomb lattice of a given spatial extent.
     Periodic boundary conditions can also be imposed.
     Sites are returned at the 2b Wyckoff positions.
@@ -395,6 +433,9 @@ def Honeycomb(extent, *, pbc: Union[bool, Sequence[bool]] = True) -> Lattice:
              This parameter can also be a list of booleans with same length as
              the parameter `length`, in which case each dimension will have
              PBC/OBC depending on the corresponding entry of `pbc`.
+        kwargs: Additional keyword arguments are passed on to the constructor of
+            :ref:`netket.graph.Lattice`.
+
 
     Example:
         Construct a honeycomb lattice with 3 × 3 unit cells:
@@ -405,11 +446,14 @@ def Honeycomb(extent, *, pbc: Union[bool, Sequence[bool]] = True) -> Lattice:
         18
     """
     return _hexagonal_general(
-        extent, site_offsets=[[0.5, 0.5 / 3 ** 0.5], [1, 1 / 3 ** 0.5]], pbc=pbc
+        extent,
+        site_offsets=[[0.5, 0.5 / 3 ** 0.5], [1, 1 / 3 ** 0.5]],
+        pbc=pbc,
+        **kwargs,
     )
 
 
-def Kagome(extent, *, pbc: Union[bool, Sequence[bool]] = True) -> Lattice:
+def Kagome(extent, *, pbc: Union[bool, Sequence[bool]] = True, **kwargs) -> Lattice:
     r"""Constructs a kagome lattice of a given spatial extent.
     Periodic boundary conditions can also be imposed.
     Sites are returned at the 3c Wyckoff positions.
@@ -422,6 +466,9 @@ def Kagome(extent, *, pbc: Union[bool, Sequence[bool]] = True) -> Lattice:
              This parameter can also be a list of booleans with same length as
              the parameter `length`, in which case each dimension will have
              PBC/OBC depending on the corresponding entry of `pbc`.
+        kwargs: Additional keyword arguments are passed on to the constructor of
+            :ref:`netket.graph.Lattice`.
+
 
     Example:
         Construct a kagome lattice with 3 × 3 unit cells:
@@ -435,4 +482,5 @@ def Kagome(extent, *, pbc: Union[bool, Sequence[bool]] = True) -> Lattice:
         extent,
         site_offsets=[[0.5, 0], [0.25, 0.75 ** 0.5 / 2], [0.75, 0.75 ** 0.5 / 2]],
         pbc=pbc,
+        **kwargs,
     )
