@@ -12,6 +12,9 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+import numpy as np
+import jax.numpy as jnp
+
 from netket.utils.dispatch import parametric
 
 from ._abstract_operator import AbstractOperator
@@ -63,6 +66,8 @@ class WrappedOperator(AbstractOperator):
                 return Squared(self)
             else:
                 return self._op__matmul__(other)
+        if isinstance(other, np.ndarray) or isinstance(other, jnp.ndarray):
+            return self._op__matmul__(other)
         else:
             return NotImplemented
 
