@@ -14,17 +14,15 @@
 
 import abc
 from typing import Optional, Union, Tuple, Callable, Iterator
-from functools import partial
 
-import jax
 import numpy as np
-
 from flax import linen as nn
 from jax import numpy as jnp
 
 from netket import jax as nkjax
 from netket.hilbert import AbstractHilbert
 from netket.utils import mpi, get_afun_if_module, wrap_afun
+from netket.utils.deprecation import deprecated
 from netket.utils.types import PyTree, DType, SeedT
 from netket.jax import HashablePartial
 from netket.utils import struct, numbers
@@ -50,23 +48,23 @@ class Sampler(abc.ABC):
     """
     Abstract base class for all samplers.
 
-    It contains the fields that all of them should posses, defining the common
+    It contains the fields that all of them should possess, defining the common
     API.
-    Note that fields marked with pytree_node=False are treated as static arguments
+    Note that fields marked with `pytree_node=False` are treated as static arguments
     when jitting.
     """
 
     hilbert: AbstractHilbert = struct.field(pytree_node=False)
-    """Hilbert space to be sampled."""
+    """The Hilbert space to sample."""
 
     n_chains_per_rank: int = struct.field(pytree_node=False, default=None)
     """Number of independent chains on every MPI rank."""
 
     machine_pow: int = struct.field(default=2)
-    """Exponent of the pdf sampled."""
+    """The power to which the machine should be exponentiated to generate the pdf."""
 
     dtype: DType = struct.field(pytree_node=False, default=np.float64)
-    """DType of the computed samples."""
+    """The dtype of the states sampled."""
 
     def __pre_init__(
         self, hilbert: AbstractHilbert, n_chains: Optional[int] = None, **kwargs
@@ -75,39 +73,37 @@ class Sampler(abc.ABC):
         Construct a Monte Carlo sampler.
 
         Args:
-            hilbert: Hilbert space to be sampled
-            n_chains: The total number of independent chains across all MPI ranks
-            n_chains_per_rank: Number of independent chains on every MPI rank
-            machine_pow: Exponent of the pdf sampled
-            dtype: DType of the computed samples
+            hilbert: The Hilbert space to sample.
+            n_chains: The total number of independent chains across all MPI ranks. Either specify this or `n_chains_per_rank`.
+            n_chains_per_rank: Number of independent chains on every MPI rank (default = 1).
+            machine_pow: The power to which the machine should be exponentiated to generate the pdf (default = 2).
+            dtype: The dtype of the states sampled (default = np.float64).
         """
 
-        # Default number of total chains
         if "n_chains_per_rank" in kwargs:
             if n_chains is not None:
                 raise ValueError(
                     "Cannot specify both `n_chains` and `n_chains_per_rank`"
                 )
         else:
-
-            # DEFAULT VALUE
             if n_chains is None:
-                n_chains = 16
+                # Default value
+                n_chains_per_rank = 1
+            else:
+                n_chains_per_rank = max(int(np.ceil(n_chains / mpi.n_nodes)), 1)
+                if mpi.n_nodes > 1 and mpi.rank == 0:
+                    if n_chains_per_rank * mpi.n_nodes != n_chains:
+                        import warnings
 
-            n_chains_per_rank = max(int(np.ceil(n_chains / mpi.n_nodes)), 1)
-            if mpi.n_nodes > 1 and mpi.rank == 0:
-                if n_chains_per_rank * mpi.n_nodes != n_chains:
-                    import warnings
-
-                    warnings.warn(
-                        f"Using {n_chains_per_rank} chains per rank among {mpi.n_nodes} ranks (total="
-                        f"{n_chains_per_rank*mpi.n_nodes} instead of n_chains={n_chains})."
-                        f"To directly control the number of chains on every rank, specify "
-                        f"`n_chains_per_rank` when constructing the sampler. "
-                        f"To silence this warning, either use `n_chains_per_rank` or use `n_chains` "
-                        f"that is a multiple of the number of mpi ranks.",
-                        category=UserWarning,
-                    )
+                        warnings.warn(
+                            f"Using {n_chains_per_rank} chains per rank among {mpi.n_nodes} ranks "
+                            f"(total={n_chains_per_rank * mpi.n_nodes} instead of n_chains={n_chains}). "
+                            f"To directly control the number of chains on every rank, specify "
+                            f"`n_chains_per_rank` when constructing the sampler. "
+                            f"To silence this warning, either use `n_chains_per_rank` or use `n_chains` "
+                            f"that is a multiple of the number of MPI ranks.",
+                            category=UserWarning,
+                        )
 
             kwargs["n_chains_per_rank"] = n_chains_per_rank
 
@@ -123,7 +119,7 @@ class Sampler(abc.ABC):
 
         # workaround Jax bug under pmap
         # might be removed in the future
-        if not type(self.machine_pow) == object:
+        if type(self.machine_pow) != object:
             if not np.issubdtype(numbers.dtype(self.machine_pow), np.integer):
                 raise ValueError(
                     f"machine_pow ({self.machine_pow}) must be a positive integer"
@@ -132,9 +128,9 @@ class Sampler(abc.ABC):
     @property
     def n_chains(self) -> int:
         """
-        The total number of chains across all MPI ranks.
+        The total number of independent chains across all MPI ranks.
 
-        If you are not using MPI, this is equal to `n_chains_per_rank`
+        If you are not using MPI, this is equal to `n_chains_per_rank`.
         """
         return self.n_chains_per_rank * mpi.n_nodes
 
@@ -143,7 +139,7 @@ class Sampler(abc.ABC):
         r"""
         The batch size of the configuration $\sigma$ used by this sampler.
 
-        In general, it is equivalent to :attr:`~Sampler.n_chains`.
+        In general, it is equivalent to :attr:`~Sampler.n_chains_per_rank`.
         """
         return self.n_chains_per_rank
 
@@ -159,16 +155,18 @@ class Sampler(abc.ABC):
 
     def log_pdf(self, model: Union[Callable, nn.Module]) -> Callable:
         """
-        Returns a closure with the log_pdf function encoded by this sampler.
-
-        Note: the result is returned as an HashablePartial so that the closure
-        does not trigger recompilation.
+        Returns a closure with the log-pdf function encoded by this sampler.
 
         Args:
-            model: The machine, or apply_fun
+            model: A Flax module or callable with the forward pass of the log-pdf.
+                If it is a callable, it should have the signature :code:`f(parameters, σ) -> jnp.ndarray`.
 
         Returns:
-            the log probability density function
+            The log-probability density function.
+
+        Note:
+            The result is returned as a `HashablePartial` so that the closure
+            does not trigger recompilation.
         """
         apply_fun = get_afun_if_module(model)
         log_pdf = HashablePartial(
@@ -189,17 +187,18 @@ class Sampler(abc.ABC):
         If you want reproducible samples, you should specify `seed`, otherwise the state
         will be initialised randomly.
 
-        If running across several MPI processes, all sampler_states are guaranteed to be
+        If running across several MPI processes, all `sampler_state`s are guaranteed to be
         in a different (but deterministic) state.
         This is achieved by first reducing (summing) the seed provided to every MPI rank,
-        then generating n_rank seeds starting from the reduced one, and every rank is
+        then generating `n_rank` seeds starting from the reduced one, and every rank is
         initialized with one of those seeds.
 
-        The resulting state is guaranteed to be a frozen python dataclass (in particular,
-        a flax's dataclass), and it can be serialized using Flax serialization methods.
+        The resulting state is guaranteed to be a frozen Python dataclass (in particular,
+        a Flax dataclass), and it can be serialized using Flax serialization methods.
 
         Args:
-            machine: a Flax module or callable with the forward pass of the log-pdf.
+            machine: A Flax module or callable with the forward pass of the log-pdf.
+                If it is a callable, it should have the signature :code:`f(parameters, σ) -> jnp.ndarray`.
             parameters: The PyTree of parameters of the model.
             seed: An optional seed or jax PRNGKey. If not specified, a random seed will be used.
 
@@ -222,46 +221,20 @@ class Sampler(abc.ABC):
         Resets the state of the sampler. To be used every time the parameters are changed.
 
         Args:
-            machine: a Flax module or callable with the forward pass of the log-pdf.
+            machine: A Flax module or callable with the forward pass of the log-pdf.
+                If it is a callable, it should have the signature :code:`f(parameters, σ) -> jnp.ndarray`.
             parameters: The PyTree of parameters of the model.
-            state: The current state of the sampler. If it's not provided, it will be constructed
+            state: The current state of the sampler. If not specified, it will be constructed
                 by calling :code:`sampler.init_state(machine, parameters)` with a random seed.
 
         Returns:
             A valid sampler state.
         """
         if state is None:
-            state = sampler_state(sampler, machine, parameters)
+            state = sampler.init_state(machine, parameters)
 
         return sampler._reset(wrap_afun(machine), parameters, state)
 
-    def sample_next(
-        sampler,
-        machine: Union[Callable, nn.Module],
-        parameters: PyTree,
-        state: Optional[SamplerState] = None,
-    ) -> Tuple[jnp.ndarray, SamplerState]:
-        """
-        Samples the next state in the markov chain.
-
-        Args:
-            machine: a Flax module or callable apply function with the forward pass of the log-pdf.
-            parameters: The PyTree of parameters of the model.
-            state: The current state of the sampler. If it's not provided, it will be constructed
-                by calling :code:`sampler.reset(machine, parameters)` with a random seed.
-
-        Returns:
-            state: The new state of the sampler
-            σ: The next batch of samples.
-        """
-        # Note: the return order is inverted wrt `.sample` because when called inside of
-        # a scan function the first returned argument should be the state.
-
-        if state is None:
-            state = sampler_state(sampler, machine, parameters)
-
-        return sampler._sample_next(wrap_afun(machine), parameters, state)
-
     def sample(
         sampler,
         machine: Union[Callable, nn.Module],
@@ -271,86 +244,107 @@ class Sampler(abc.ABC):
         chain_length: int = 1,
     ) -> Tuple[jnp.ndarray, SamplerState]:
         """
-        Samples chain_length elements along the chains.
+        Samples `chain_length` batches of samples along the chains.
 
         Arguments:
-            sampler: The Monte Carlo sampler.
-            machine: The model or callable to sample from (if it's a function it should have
-                the signature :code:`f(parameters, σ) -> jnp.ndarray`).
+            machine: A Flax module or callable with the forward pass of the log-pdf.
+                If it is a callable, it should have the signature :code:`f(parameters, σ) -> jnp.ndarray`.
             parameters: The PyTree of parameters of the model.
-            state: current state of the sampler. If None, then initialises it.
-            chain_length: (default=1), the length of the chains.
+            state: The current state of the sampler. If not specified, then initialize and reset it.
+            chain_length: The length of the chains (default = 1).
 
         Returns:
-            σ: The next batch of samples.
-            state: The new state of the sampler
+            σ: The generated batches of samples.
+            state: The new state of the sampler.
         """
+        if state is None:
+            state = sampler.reset(machine, parameters)
 
-        return sample(
-            sampler, machine, parameters, state=state, chain_length=chain_length
+        return sampler._sample_chain(
+            wrap_afun(machine), parameters, state, chain_length
         )
 
-    def _sample_chain(
+    def samples(
         sampler,
         machine: Union[Callable, nn.Module],
         parameters: PyTree,
+        *,
+        state: Optional[SamplerState] = None,
+        chain_length: int = 1,
+    ) -> Iterator[jnp.ndarray]:
+        """
+        Returns a generator sampling `chain_length` batches of samples along the chains.
+
+        Arguments:
+            machine: A Flax module or callable with the forward pass of the log-pdf.
+                If it is a callable, it should have the signature :code:`f(parameters, σ) -> jnp.ndarray`.
+            parameters: The PyTree of parameters of the model.
+            state: The current state of the sampler. If not specified, then initialize and reset it.
+            chain_length: The length of the chains (default = 1).
+        """
+        if state is None:
+            state = sampler.reset(machine, parameters)
+
+        machine = wrap_afun(machine)
+
+        for i in range(chain_length):
+            samples, state = sampler._sample_chain(machine, parameters, state, 1)
+            yield samples[0, :, :]
+
+    @abc.abstractmethod
+    def _sample_chain(
+        sampler,
+        machine: nn.Module,
+        parameters: PyTree,
         state: SamplerState,
         chain_length: int,
     ) -> Tuple[jnp.ndarray, SamplerState]:
         """
-        Samples chain_length elements along the chains.
+        Implementation of `sample` for subclasses of `Sampler`.
 
-        In general this should not be overridden unless you want to modify the logic by which
-        the whole sampling is performed.
-        If using Jax, this function should be jitted
+        If you subclass `Sampler`, you should override this and not `sample`
+        itself, because `sample` contains some common logic.
+
+        If using Jax, this function should be jitted.
 
         Arguments:
-            sampler: The Monte Carlo sampler.
-            machine: The model or callable to sample from (if it's a function it should have
-                the signature :code:`f(parameters, σ) -> jnp.ndarray`).
+            machine: A Flax module with the forward pass of the log-pdf.
             parameters: The PyTree of parameters of the model.
-            state: current state of the sampler. If None, then initialises it.
-            chain_length: (default=1), the length of the chains.
+            state: The current state of the sampler.
+            chain_length: The length of the chains.
 
         Returns:
-            σ: The next batch of samples.
-            state: The new state of the sampler
+            σ: The generated batches of samples.
+            state: The new state of the sampler.
         """
-        return _sample_chain(sampler, machine, parameters, state, chain_length)
 
     @abc.abstractmethod
     def _init_state(sampler, machine, params, seed) -> SamplerState:
         """
-        Implementation of init_state for subclasses of Sampler.
+        Implementation of `init_state` for subclasses of `Sampler`.
 
-        If you sub-class Sampler, you should define this and not init_state
-        itself, because init_state contains some common logic.
+        If you subclass `Sampler`, you should override this and not `init_state`
+        itself, because `init_state` contains some common logic.
         """
-        raise NotImplementedError("init_state Not Implemented")
 
     @abc.abstractmethod
     def _reset(sampler, machine, parameters, state):
         """
-        Implementation of reset for subclasses of Sampler.
+        Implementation of `reset` for subclasses of `Sampler`.
 
-        If you sub-class Sampler, you should define _reset and not reset
-        itself, because reset contains some common logic.
+        If you subclass `Sampler`, you should override this and not `reset`
+        itself, because `reset` contains some common logic.
         """
-        raise NotImplementedError("reset Not Implemented")
-
-    @abc.abstractmethod
-    def _sample_next(sampler, machine, parameters, state=None):
-        """
-        Implementation of sample_next for subclasses of Sampler.
-
-        If you sub-class Sampler, you should define _sample_next and not sample_next
-        itself, because reset contains some common logic.
-        """
-        raise NotImplementedError("sample_next Not Implemented")
 
 
+@deprecated(
+    "The module function `sampler_state` is deprecated in favor of the class method `init_state`."
+)
 def sampler_state(
-    sampler: Sampler, machine: Union[Callable, nn.Module], parameters: PyTree
+    sampler: Sampler,
+    machine: Union[Callable, nn.Module],
+    parameters: PyTree,
+    seed: Optional[SeedT] = None,
 ) -> SamplerState:
     """
     Creates the structure holding the state of the sampler.
@@ -358,16 +352,19 @@ def sampler_state(
     If you want reproducible samples, you should specify `seed`, otherwise the state
     will be initialised randomly.
 
-    If running across several MPI processes, all sampler_states are guaranteed to be
+    If running across several MPI processes, all `sampler_state`s are guaranteed to be
     in a different (but deterministic) state.
-
     This is achieved by first reducing (summing) the seed provided to every MPI rank,
-    then generating n_rank seeds starting from the reduced one, and every rank is
+    then generating `n_rank` seeds starting from the reduced one, and every rank is
     initialized with one of those seeds.
 
+    The resulting state is guaranteed to be a frozen Python dataclass (in particular,
+    a Flax dataclass), and it can be serialized using Flax serialization methods.
+
     Args:
         sampler: The Monte Carlo sampler.
-        machine: a Flax module or callable with the forward pass of the log-pdf.
+        machine: A Flax module or callable with the forward pass of the log-pdf.
+            If it is a callable, it should have the signature :code:`f(parameters, σ) -> jnp.ndarray`.
         parameters: The PyTree of parameters of the model.
         seed: An optional seed or jax PRNGKey. If not specified, a random seed will be used.
 
@@ -375,9 +372,12 @@ def sampler_state(
         The structure holding the state of the sampler. In general you should not expect
         it to be in a valid state, and should reset it before use.
     """
-    return sampler.init_state(machine, parameters)
+    return sampler.init_state(machine, parameters, seed)
 
 
+@deprecated(
+    "The module function `reset` is deprecated in favor of the class method `reset`."
+)
 def reset(
     sampler: Sampler,
     machine: Union[Callable, nn.Module],
@@ -389,40 +389,21 @@ def reset(
 
     Args:
         sampler: The Monte Carlo sampler.
-        machine: a Flax module or Callable with the forward pass of the log-pdf.
+        machine: A Flax module or callable with the forward pass of the log-pdf.
+            If it is a callable, it should have the signature :code:`f(parameters, σ) -> jnp.ndarray`.
         parameters: The PyTree of parameters of the model.
-        state: The current state of the sampler. If it's not provided, it will be constructed
+        state: The current state of the sampler. If not specified, it will be constructed
             by calling :code:`sampler.init_state(machine, parameters)` with a random seed.
 
     Returns:
         A valid sampler state.
     """
-    sampler.reset(machine, parameters, state)
-
-
-def sample_next(
-    sampler: Sampler,
-    machine: Union[Callable, nn.Module],
-    parameters: PyTree,
-    state: Optional[SamplerState] = None,
-) -> Tuple[jnp.ndarray, SamplerState]:
-    """
-    Samples the next state in the markov chain.
-
-    Args:
-        sampler: The Monte Carlo sampler.
-        machine: a Flax module or callable with the forward pass of the log-pdf.
-        parameters: The PyTree of parameters of the model.
-        state: The current state of the sampler. If it's not provided, it will be constructed
-            by calling :code:`sampler.reset(machine, parameters)` with a random seed.
-
-    Returns:
-        state: The new state of the sampler
-        σ: The next batch of samples.
-    """
-    return sampler.sample_next(machine, parameters, state)
+    return sampler.reset(machine, parameters, state)
 
 
+@deprecated(
+    "The module function `sample` is deprecated in favor of the class method `sample`."
+)
 def sample(
     sampler: Sampler,
     machine: Union[Callable, nn.Module],
@@ -432,63 +413,26 @@ def sample(
     chain_length: int = 1,
 ) -> Tuple[jnp.ndarray, SamplerState]:
     """
-    Samples chain_length elements along the chains.
+    Samples `chain_length` batches of samples along the chains.
 
     Arguments:
         sampler: The Monte Carlo sampler.
-        machine: The model or Callable to sample from (if it's a function it should have
-            the signature :code:`f(parameters, σ) -> jnp.ndarray`).
+        machine: A Flax module or callable with the forward pass of the log-pdf.
+            If it is a callable, it should have the signature :code:`f(parameters, σ) -> jnp.ndarray`.
         parameters: The PyTree of parameters of the model.
-        state: current state of the sampler. If None, then initialises it.
-        chain_length: (default=1), the length of the chains.
+        state: The current state of the sampler. If not specified, then initialize and reset it.
+        chain_length: The length of the chains (default = 1).
 
     Returns:
-        σ: The next batch of samples.
-        state: The new state of the sampler
+        σ: The generated batches of samples.
+        state: The new state of the sampler.
     """
-    if state is None:
-        state = sampler.reset(machine, parameters, state)
-
-    return sampler._sample_chain(machine, parameters, state, chain_length)
-
-
-@partial(jax.jit, static_argnums=(1, 4))
-def _sample_chain(
-    sampler,
-    machine: Union[Callable, nn.Module],
-    parameters: PyTree,
-    state: SamplerState,
-    chain_length: int,
-) -> Tuple[jnp.ndarray, SamplerState]:
-    """
-    Samples chain_length elements along the chains.
-
-    Internal method used for jitting calls
-
-    Arguments:
-        sampler: The Monte Carlo sampler.
-        machine: The model or Callable to sample from (if it's a function it should have
-            the signature :code:`f(parameters, σ) -> jnp.ndarray`).
-        parameters: The PyTree of parameters of the model.
-        state: current state of the sampler. If None, then initialises it.
-        chain_length: (default=1), the length of the chains.
-
-    Returns:
-        σ: The next batch of samples.
-        state: The new state of the sampler
-    """
-    _sample_next = lambda state, _: sampler.sample_next(machine, parameters, state)
-
-    state, samples = jax.lax.scan(
-        _sample_next,
-        state,
-        xs=None,
-        length=chain_length,
-    )
-
-    return samples, state
+    return sampler.sample(machine, parameters, state=state, chain_length=chain_length)
 
 
+@deprecated(
+    "The module function `samples` is deprecated in favor of the class method `samples`."
+)
 def samples(
     sampler: Sampler,
     machine: Union[Callable, nn.Module],
@@ -496,21 +440,18 @@ def samples(
     *,
     state: Optional[SamplerState] = None,
     chain_length: int = 1,
-) -> Iterator[np.ndarray]:
+) -> Iterator[jnp.ndarray]:
     """
-    Returns a generator sampling chain_length elements along the chains.
+    Returns a generator sampling `chain_length` batches of samples along the chains.
 
     Arguments:
         sampler: The Monte Carlo sampler.
-        machine: The model or Callable to sample from (if it's a function it should have
-            the signature :code:`f(parameters, σ) -> jnp.ndarray`).
+        machine: A Flax module or callable with the forward pass of the log-pdf.
+            If it is a callable, it should have the signature :code:`f(parameters, σ) -> jnp.ndarray`.
         parameters: The PyTree of parameters of the model.
-        state: current state of the sampler. If None, then initialises it.
-        chain_length: (default=1), the length of the chains.
+        state: The current state of the sampler. If not specified, then initialize and reset it.
+        chain_length: The length of the chains (default = 1).
     """
-    if state is None:
-        state = sampler.reset(machine, parameters, state)
-
-    for i in range(chain_length):
-        samples, state = sampler._sample_chain(machine, parameters, state, 1)
-        yield samples[0, :, :]
+    yield from sampler.samples(
+        machine, parameters, state=state, chain_length=chain_length
+    )
