@@ -113,7 +113,7 @@ def test_vmc_construction_vstate():
 
     op = nk.optimizer.Sgd(learning_rate=0.05)
 
-    driver = nk.Vmc(ha, op, sa, nk.models.RBM(), n_samples=1000, seed=SEED)
+    driver = nk.VMC(ha, op, sa, nk.models.RBM(), n_samples=1000, seed=SEED)
 
     driver.run(1)
 
@@ -284,3 +284,11 @@ def test_vmc_sr_legacy_api():
         driver = nk.VMC(
             ha, op, variational_state=vs, sr=sr_config, preconditioner=sr_config
         )
+
+
+def test_deprecated_vmc_name():
+    ha, sx, ma, sa, driver = _setup_vmc()
+    op = nk.optimizer.Sgd(learning_rate=0.05)
+
+    with pytest.warns(FutureWarning):
+        driver = nk.Vmc(ha, op, sa, nk.models.RBM(), n_samples=1000, seed=SEED)
